@@ -21,20 +21,23 @@
 /**
  * SECTION:element-siddec
  *
- * This element decodes .sid files to raw audio. .sid files are in fact
- * small Commodore 64 programs that are executed on an emulated 6502 CPU and a
+ * This element decodes .sid files to raw audio. .sid files are in fact 
+ * small Commodore 64 programs that are executed on an emulated 6502 CPU and a 
  * MOS 6581 sound chip.
- *
+ * 
  * This plugin will first load the complete program into memory before starting
  * the emulator and producing output.
- *
+ * 
  * Seeking is not (and cannot be) implemented.
- *
- * ## Example pipelines
- *
+ * 
+ * <refsect2>
+ * <title>Example pipelines</title>
  * |[
- * gst-launch-1.0 -v filesrc location=Hawkeye.sid ! siddec ! audioconvert ! audioresample ! autoaudiosink
- * ]| Decode a sid file and play it back.
+ * gst-launch -v filesrc location=Hawkeye.sid ! siddec ! audioconvert ! alsasink
+ * ]| Decode a sid file and play back the audio using alsasink.
+ * </refsect2>
+ *
+ * Last reviewed on 2006-12-30 (0.10.5)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -74,7 +77,7 @@ static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("audio/x-sid")
     );
 
-#define FORMATS "{ " GST_AUDIO_NE(S16) "," GST_AUDIO_NE(U16) ", S8, U8 }"
+#define FORMATS "{ "GST_AUDIO_NE(S16)","GST_AUDIO_NE(U16)", S8, U8 }"
 
 static GstStaticPadTemplate src_templ = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -144,8 +147,6 @@ static void gst_siddec_set_property (GObject * object, guint prop_id,
 
 #define gst_siddec_parent_class parent_class
 G_DEFINE_TYPE (GstSidDec, gst_siddec, GST_TYPE_ELEMENT);
-GST_ELEMENT_REGISTER_DEFINE (siddec, "siddec", GST_RANK_PRIMARY,
-    GST_TYPE_SIDDEC);
 
 static void
 gst_siddec_class_init (GstSidDecClass * klass)
@@ -199,14 +200,13 @@ gst_siddec_class_init (GstSidDecClass * klass)
       "Codec/Decoder/Audio", "Use libsidplay to decode SID audio tunes",
       "Wim Taymans <wim.taymans@gmail.com>");
 
-  gst_element_class_add_static_pad_template (gstelement_class, &src_templ);
-  gst_element_class_add_static_pad_template (gstelement_class, &sink_templ);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_templ));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_templ));
 
   GST_DEBUG_CATEGORY_INIT (gst_siddec_debug, "siddec", 0,
       "C64 sid song player");
-
-  gst_type_mark_as_plugin_api (GST_TYPE_SID_CLOCK, static_cast<GstPluginAPIFlags>(0));
-  gst_type_mark_as_plugin_api (GST_TYPE_SID_MEMORY, static_cast<GstPluginAPIFlags>(0));
 }
 
 static void
@@ -398,7 +398,7 @@ play_loop (GstPad * pad)
   GstSidDec *siddec;
   GstBuffer *out;
   GstMapInfo outmap;
-  gint64 value, offset, time = 0;
+  gint64 value, offset, time;
   GstFormat format;
 
   siddec = GST_SIDDEC (gst_pad_get_parent (pad));
@@ -412,29 +412,29 @@ play_loop (GstPad * pad)
 
   /* get offset in samples */
   format = GST_FORMAT_DEFAULT;
-  if (gst_siddec_src_convert (siddec->srcpad,
-          GST_FORMAT_BYTES, siddec->total_bytes, &format, &offset))
-    GST_BUFFER_OFFSET (out) = offset;
+  gst_siddec_src_convert (siddec->srcpad,
+      GST_FORMAT_BYTES, siddec->total_bytes, &format, &offset);
+  GST_BUFFER_OFFSET (out) = offset;
 
   /* get current timestamp */
   format = GST_FORMAT_TIME;
-  if (gst_siddec_src_convert (siddec->srcpad,
-          GST_FORMAT_BYTES, siddec->total_bytes, &format, &time))
-    GST_BUFFER_TIMESTAMP (out) = time;
+  gst_siddec_src_convert (siddec->srcpad,
+      GST_FORMAT_BYTES, siddec->total_bytes, &format, &time);
+  GST_BUFFER_TIMESTAMP (out) = time;
 
   /* update position and get new timestamp to calculate duration */
   siddec->total_bytes += siddec->blocksize;
 
   /* get offset in samples */
   format = GST_FORMAT_DEFAULT;
-  if (gst_siddec_src_convert (siddec->srcpad,
-          GST_FORMAT_BYTES, siddec->total_bytes, &format, &value))
-    GST_BUFFER_OFFSET_END (out) = value;
+  gst_siddec_src_convert (siddec->srcpad,
+      GST_FORMAT_BYTES, siddec->total_bytes, &format, &value);
+  GST_BUFFER_OFFSET_END (out) = value;
 
   format = GST_FORMAT_TIME;
-  if (gst_siddec_src_convert (siddec->srcpad,
-          GST_FORMAT_BYTES, siddec->total_bytes, &format, &value))
-    GST_BUFFER_DURATION (out) = value - time;
+  gst_siddec_src_convert (siddec->srcpad,
+      GST_FORMAT_BYTES, siddec->total_bytes, &format, &value);
+  GST_BUFFER_DURATION (out) = value - time;
 
   if ((ret = gst_pad_push (siddec->srcpad, out)) != GST_FLOW_OK)
     goto pause;
@@ -447,17 +447,19 @@ done:
   /* ERRORS */
 pause:
   {
+    const gchar *reason = gst_flow_get_name (ret);
+
     if (ret == GST_FLOW_EOS) {
       /* perform EOS logic, FIXME, segment seek? */
       gst_pad_push_event (pad, gst_event_new_eos ());
     } else if (ret < GST_FLOW_EOS || ret == GST_FLOW_NOT_LINKED) {
       /* for fatal errors we post an error message */
-      GST_ELEMENT_FLOW_ERROR (siddec, ret);
+      GST_ELEMENT_ERROR (siddec, STREAM, FAILED,
+          (NULL), ("streaming task paused, reason %s", reason));
       gst_pad_push_event (pad, gst_event_new_eos ());
     }
 
-    GST_INFO_OBJECT (siddec, "pausing task, reason: %s",
-        gst_flow_get_name (ret));
+    GST_INFO_OBJECT (siddec, "pausing task, reason: %s", reason);
     gst_pad_pause_task (pad);
     goto done;
   }
@@ -770,11 +772,12 @@ gst_siddec_get_property (GObject * object, guint prop_id, GValue * value,
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  return GST_ELEMENT_REGISTER (siddec, plugin);
+  return gst_element_register (plugin, "siddec", GST_RANK_PRIMARY,
+      GST_TYPE_SIDDEC);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    sid,
+    siddec,
     "Uses libsidplay to decode .sid files",
     plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
